﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

// Glyph image at: http://futurama.wikia.com/wiki/Alienese

using System.Drawing.Drawing2D;

namespace howto_alienese
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        // Dictionary to map characters to glyph buttons.
        private Dictionary<char, Image> GlyphDict =
            new Dictionary<char, Image>();

        private void Form1_Load(object sender, EventArgs e)
        {
            // Create the decoding buttons.
            for (char ch = 'A'; ch <= 'M'; ch++)
                MakeButton(GlyphDict, flpButtons1, ch.ToString());
            for (char ch = 'N'; ch <= 'Z'; ch++)
                MakeButton(GlyphDict, flpButtons2, ch.ToString());
            for (char ch = '0'; ch <= '9'; ch++)
                MakeButton(GlyphDict, flpButtons3, ch.ToString());
            foreach (char ch in "\'!“”:-.; ")
                MakeButton(GlyphDict, flpButtons4, ch.ToString());

            Button btnCl = new Button();
            btnCl.Font = this.Font;
            btnCl.Parent = flpButtons4;
            btnCl.Size = new Size(54, 24);
            btnCl.Text = "CL";
            btnCl.Click += btnCl_Click;

            Button btnBs = new Button();
            btnBs.Font = this.Font;
            btnBs.Parent = flpButtons4;
            btnBs.Size = new Size(54, 24);
            btnBs.Text = "BS";
            btnBs.Click += btnBs_Click;
        }

        // Make a button.
        private const int BTN_WID = 24;
        private void MakeButton(
            Dictionary<char, Image> glyph_dict,
            Control parent, string text)
        {
            // Make the button.
            Button btn = new Button();
            btn.Parent = parent;
            btn.Size = new Size(BTN_WID, BTN_WID);
            btn.Tag = text;
            btn.Font = this.Font;
            btn.Click += btnDecode_Click;
            ttip.SetToolTip(btn, text);

            // Get the button's image.
            string name = ResourceName(text);
            Bitmap bm = Properties.Resources.ResourceManager.GetObject(name) as Bitmap;
            Bitmap new_bm = new Bitmap(BTN_WID, BTN_WID);
            using (Graphics gr = Graphics.FromImage(new_bm))
            {
                gr.InterpolationMode = InterpolationMode.High;
                gr.Clear(Color.White);
                Point[] dest_points =
                {
                    new Point(2, 2),
                    new Point(btn.ClientSize.Width - 3, 2),
                    new Point(2, btn.ClientSize.Height - 3),
                };
                gr.DrawImage(bm, dest_points);
            }
            btn.Image = new_bm;

            // Save the image in the glyph dictionary.
            glyph_dict.Add(text[0], new_bm);
        }

        // Convert a character into its resource name.
        private string ResourceName(string text)
        {
            char ch = text[0];
            if ((ch >= 'A') && (ch <= 'Z')) return text;
            if ((ch >= '0') && (ch <= '9')) return "_" + text;
            switch (ch)
            {
                case '\'': return "Apostrophe";
                case '!': return "Bang";
                case '“': return "OpenQuote";
                case '”': return "OpenQuote";
                case ':': return "Colon";
                case '-': return "Dash";
                case '.': return "Period";
                case ';': return "SemiColon";
                case ' ': return "Space";
                default: return null;
            }
        }

        // Clear the decoded message.
        private void btnCl_Click(object sender, EventArgs e)
        {
            txtDecode.Clear();
        }

        // Remove the last letter from the decoded message.
        private void btnBs_Click(object sender, EventArgs e)
        {
            int len = txtDecode.Text.Length;
            if (len > 0)
                txtDecode.Text = txtDecode.Text.Substring(0, len - 1);
        }

        // Add a character to the decoded message.
        private void btnDecode_Click(object sender, EventArgs e)
        {
            Button btn = sender as Button;
            txtDecode.AppendText(btn.Tag as string);
        }

        // Add a character to the decoded message.
        private void txtEncode_TextChanged(object sender, EventArgs e)
        {
            flpEncoding.Controls.Clear();
            foreach (char ch in txtEncode.Text.ToUpper())
            {
                if (GlyphDict.ContainsKey(ch))
                {
                    PictureBox pic = new PictureBox();
                    pic.SizeMode = PictureBoxSizeMode.AutoSize;
                    pic.Image = GlyphDict[ch];
                    pic.Parent = flpEncoding;
                }
            }
        }

        // Load a bitmap without locking it.
        private Bitmap LoadBitmapUnlocked(string file_name)
        {
            using (Bitmap bm = new Bitmap(file_name))
            {
                return new Bitmap(bm);
            }
        }
    }
}
